import click
from click import echo, style

from slot.config import Config
from slot.store import Store


@click.command()
def list_stores():
    stores = [Store(name) for name in Config().stores.keys()]

    for store in stores:
        echo(
            '{} -> {}{}'.format(
                style(
                    store.name,
                    fg='cyan',
                    bold=True,
                ),
                style(
                    'installed' if store.registered else 'not installed',
                    fg='green' if store.registered else 'red',
                    bold=True,
                ),
                style(
                    f' [{store.selected_option}]' if store.registered else '',
                    fg='blue',
                    bold=True,
                ),
            ),
        )



import os

import click

from slot.store import Store


@click.command()
@click.argument('store_name')
@click.argument('file_name')
@click.option('-n', '--name', type=click.STRING, default=None,
              help='Name of the option this file becomes')
@click.option('-s', '--silent', type=click.BOOL, help='Disable user interaction')
def ingest(store_name, file_name, name, silent):
    store = Store(store_name)

    new_name = name or os.path.basename(file_name)

    if name is None and silent is False:
        if not click.confirm(
            'Is the name {} okay for this option?'.format(
                click.style(
                    new_name,
                    bold=True,
                ),
            ),
        ):
            new_name = click.prompt('Please choose a new name')

    store.ingest(new_name, file_name=file_name)

    click.secho('Done!', fg='green', bold=True)



import click

from slot.store import Store


@click.command()
@click.argument('store_name')
def options(store_name):
    store = Store(store_name)

    for option in store.options:
        is_selected = store.selected_option == option
        prelude = '->' if is_selected else '  '

        click.secho(
            f'{prelude} {option}',
            color='cyan' if is_selected else None,
            bold=is_selected,
        )



import click

from slot.store import Store


@click.command()
@click.argument('store_name')
@click.argument('option')
def select(store_name, option):
    store = Store(store_name)

    store.select(option)

    click.secho('Done!', fg='green', bold=True)



import click
from click import echo, style, secho

from slot.store import Store


@click.command()
@click.argument('store_name')
@click.argument('option')
def use(store_name, option):
    store = Store(store_name)

    echo('Registering {} in {}'.format(
        style(option, fg='cyan', bold=True),
        style(store_name, fg='cyan', bold=True),
    ))
    store.select(option)

    secho('Done!', fg='green', bold=True)



import os

from box import Box
from ruamel.yaml import YAML

from .constants import ROOT_DIR


yaml = YAML(typ="safe")
yaml.default_flow_style = False

file_name = 'config.yml'


class Config:
    file_path = os.path.join(ROOT_DIR, file_name)

    def __new__(self, initial_data={}):
        try:
            return Config.load()
        except FileNotFoundError:
            pass

        data = {'stores': {}}
        data.update(initial_data)
        Config.dump(Box(data))

        return Config.load()

    @classmethod
    def load(cls):
        with open(cls.file_path) as f:
            return Box(yaml.load(f.read()))

    @classmethod
    def dump(cls, data):
        with open(cls.file_path, 'w') as f:
            yaml.dump(data.to_dict(), f)



import os
from glob import glob
from shutil import move

from use_dir import use_dir
from box import Box

from .constants import STORES_DIR
from .config import Config
from .exceptions import NotRegisteredError, InvalidOptionError


class Store:
    def __init__(self, name):
        self.name = name
        self.path = os.path.join(STORES_DIR, name)

        config = Config()
        if name in config.stores.keys():
            self.target = config.stores[name].target

    @property
    def options(self):
        with use_dir(self.path):
            return glob('*')

    @property
    def registered(self):
        return self.exists and os.path.exists(self.target) and os.path.islink(self.target)

    @property
    def exists(self):
        return os.path.exists(self.path) and self.name in Config().stores.keys()

    @property
    def selected_option(self):
        return os.path.basename(os.readlink(self.target))

    def select(self, option):
        if not self.registered:
            raise NotRegisteredError(f'{self.name} is not registered')

        if option not in self.options:
            raise InvalidOptionError()

        os.unlink(self.target)
        os.symlink(os.path.join(self.path, option), self.target)

    def ingest(self, new_name, file_name=None, link=False):
        target = self.target

        if file_name is not None:
            target = file_name

        new_path = os.path.join(self.path, new_name)
        move(target, new_path)

        if link:
            os.symlink(new_path, self.target)

    def create(self, target):
        config = Config()

        os.makedirs(self.path, exist_ok=True)
        config.stores[self.name] = Box({})
        config.stores[self.name].target = target

        self.target = target

        Config.dump(config)



import click

from .stores import stores
from .list import list_stores
from .use import use
from .options import options


@click.group()
def cli():
    pass


cli.add_command(stores)
cli.add_command(use)
cli.add_command(list_stores, name='list')
cli.add_command(options)



import click
from click import echo, style, secho

from slot.store import Store


@click.command()
@click.argument('name')
@click.argument('target')
def create(name, target):
    obj = Store(name)

    if obj.registered:
        echo('Store {} is already installed!'.format(style(obj.name, fg='red', bold=True)))
        return 1

    obj.create(target)
    echo(
        'Successfully created store: {} -> {}'.format(
            style(
                obj.name,
                fg='green',
                bold=True
            ), style(
                obj.target,
                fg='green',
                bold=True
            ),
        )
    )

    if click.confirm('Do you want to move the current file into the store?'):
        new_name = click.prompt(
            'Please enter a name for this option (e.g. default)',
            confirmation_prompt=True
        )
        obj.ingest(new_name, link=True)
        secho('Done!', fg='green', bold=True)



from glob import glob

from use_dir import use_dir

from .store import Store
from .constants import STORES_DIR


def get_all_stores():
    with use_dir(STORES_DIR):
        return [Store(d) for d in glob('*')]
